/*
    tstring.h
    Template string helpers.

    MUTATE Systems 2018 (https://mutate.jp)
*/

#pragma once

template<typename T> size_t strLen(const T* sz) {
	size_t i = 0;
	while(sz[i])
		i++;
	return i;
}

template<typename T1, typename T2> int strCmp(const T1* sz1, const T2* sz2) {
    while(*sz1 || *sz2) {
		int d = int(*sz1) - int(*sz2);
		if(d) return d;
        sz1 ++;
        sz2 ++;
    }
    return 0;
}

template<typename T1, typename T2> int strNCmp(const T1* sz1, const T2* sz2, size_t len) {
    while(*sz1 || *sz2) {
		int d = int(*sz1) - int(*sz2);
		if(d) return d;
        if((-- len) == 0) return 0;
        sz1 ++;
        sz2 ++;
    }
    return 0;
}

template<typename T> const T* strChr(const T* sz, int c) {
    while(*sz) {
        if(*sz == c)
            return sz;
        sz ++;
    }
    return NULL;
}

template<typename T> const T* strRChr(const T* sz, int c) {
    auto p = sz + strLen(sz) - 1;
    while(p >= sz) {
        if(*p == c) return p;
        p --;
    }
    return NULL;
}

template<typename T1, typename T2> const T1* strRChr(const T1* sz1, const T2* sz2) {
    auto p = sz1 + strLen(sz1) - 1;
    while(p >= sz1) {
        if(strChr(sz2, *p)) return p;
        p --;
    }
    return NULL;
}

template<typename T1, typename T2> bool strBegin(const T1* sz1, const T2* sz2) {
    size_t len = strLen(sz2);
    return !strNCmp(sz1, sz2, len);
}
